#
# Copyright 2016 BMC Software, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import argparse
from tspetl import ApacheLogTool
from tspetl import DBTool
from tspetl import CSVTool
from tspetl import GitHubTool
from tspetl import JiraTool
from tspetl import LogTool
from tspetl import SalesForceTool
from tspetl import SNMPTool
from tspetl import StockTool
from tspetl import SysLogTool
from tspetl import TwitterTool
from tspetl import WeatherTool
from tspetl import XMLTool

from tspetl import APIDataSink


class TspEtlCli(object):

    def __init__(self):
        self._parser = None
        self._subparsers = None
        self._args = None

        self._tool_map = {}
        self._add_tool(ApacheLogTool())
        # self._add_tool(DBTool())
        self._add_tool(CSVTool())
        # self._add_tool(GitHubTool())
        # self._add_tool(JiraTool())
        self._add_tool(LogTool())
        # self._add_tool(SalesForceTool())
        # self._add_tool(SNMPTool())
        # self._add_tool(StockTool())
        # self._add_tool(SysLogTool())
        self._add_tool(TwitterTool())
        self._add_tool(WeatherTool())
        # self._add_tool(XMLTool())

    def _add_tool(self, tool):
        self._tool_map[tool.name] = tool

    def load_tools(self):
        """
        TODO Add capability to dynamically load the tools
        :return:
        """
        pass

    def _create_parser(self):
        self._parser = argparse.ArgumentParser(description="Tool to extract/transform/load into Pulse")
        self._subparsers = self._parser.add_subparsers(help='commands', dest='command_name')

        for key, tool in self._tool_map.iteritems():
            tool.add_parser(self._subparsers)

    def run(self):
        self._create_parser()
        args = self._parser.parse_args()
        tool = self._tool_map[args.command_name]
        sink = APIDataSink(api_host=tool.api_host, email=tool.email, api_token=tool.api_token)
        tool.handle_arguments(args)
        tool.run(sink)


def main():
    cli = TspEtlCli()
    cli.run()


if __name__ == '__main__':
    main()



#
# Copyright 2016 BMC Software, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from time import sleep
import string
from tspetl import ETLCollector
from tspetl import ETLTool
import pyowm
from tspapi import Measurement


class WeatherCollector(ETLCollector):
    def __init__(self, sink, api_key=None, cities=None):
        super(WeatherCollector, self).__init__(sink)

        self._api_key = None
        self._cities = None

        if api_key is not None:
            self._api_key = api_key
        if cities is not None:
            self._cities = cities

        self._owm = pyowm.OWM(self._api_key)

    def collect(self):
        measurements = []
        for city in self._cities:
            observation = self._owm.weather_at_place(city)
            weather = observation.get_weather()
            city = string.replace(city, ',', '_')
            city = string.replace(city, ' ', '_')
            temperature = weather.get_temperature('celsius')['temp']
            measurement = Measurement(metric='OWM_TEMPERATURE', source=city, value=temperature)
            measurements.append(measurement)

        self._sink.send_measurements(measurements=measurements)


class WeatherTool(ETLTool):
    def __init__(self):
        super(WeatherTool, self).__init__()
        self._city_names = None
        self._interval = 60
        self._api_key = None

    @property
    def name(self):
        return 'weather'

    @property
    def help(self):
        return 'Collects weather measurements from a city and optional country code. (Future Release)'

    def add_parser(self, sub_parser):
        super(WeatherTool, self).add_parser(sub_parser)
        self._parser.add_argument('-c', '--city-name', dest='city_names', action='append', metavar="city_name",
                                  required=True,
                                  help="Name of a city with an optional country code")
        self._parser.add_argument('-i', '--interval', dest='interval', action='store', metavar="seconds",
                                  required=False,
                                  help="How often to collect in each API call in seconds. Defaults to 1 minute")
        self._parser.add_argument('-k', '--api-key', dest='api_key', action='store', metavar="key", required=True,
                                  help="Open Weather Map API Key")

    def handle_arguments(self, args):
        super(WeatherTool, self).handle_arguments(args)

        if args.city_names is not None:
            print(type(args.city_names))
            self._city_names = args.city_names

        if args.interval is not None:
            self._interval = args.interval

        if args.api_key is not None:
            self._api_key = args.api_key

    def run(self, sink):
        collector = WeatherCollector(sink, cities=self._city_names, api_key=self._api_key)

        while True:
            collector.collect()
            sleep(float(self._interval))



#
# Copyright 2016 BMC Software, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from tspetl import ETLTool


class DBTool(ETLTool):
    def __init__(self):

        self._name = None
        self._password = None
        self._database = None
        self._query = None

    @property
    def name(self):
        return 'db'

    @property
    def help(self):
        return 'Import data from a relational database (Future Release)'

    def add_parser(self, parser):
        self._parser = parser.add_parser(self.name, help=self.help)
        self._parser.add_argument('-u', '--user', metavar='name',
                                  help="name of the user to connect to the database")
        self._parser.add_argument('-p', '--password', metavar='password',
                                  help="password of the user to connect to the database")
        self._parser.add_argument('-d', '--database', metavar='db_name',
                                  help="database to extract data from")
        self._parser.add_argument('-q', '--query', metavar='sql_query',
                                  help="SQL query to use to extract data")

    def handle_arguments(self, args):

        if args.name is not None:
            self._name = args.name
        pass

    def run(self, sink):
        print("Import CSV")



#
# Copyright 2016 BMC Software, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from tspapi import Measurement
from tspetl import ETLTool
from tspetl import ETLCollector
import csv
import time
import logging

logger = logging.getLogger(__name__)


class CSVCollector(ETLCollector):

    def __init__(self, sink, file_path, batch_count, skip_first_line):
        super(CSVCollector, self).__init__()

    def collect(self):
        pass


class CSVTool(ETLTool):
    def __init__(self):
        super(CSVTool, self).__init__()
        self._file_path = None
        self._batch_count = 1
        self._skip_first_line = False
        self._origin = None
        self._app_id = None
        logging.basicConfig(level=logging.DEBUG)

    @property
    def name(self):
        return 'csv'

    @property
    def help(self):
        return 'Import CSV file'

    def add_parser(self, sub_parser):
        super(CSVTool, self).add_parser(sub_parser)
        self._parser.add_argument('-f', '--file', dest='file_path', metavar="file_path",
                                  help="Path to file to import", required=False)
        self._parser.add_argument('-b', '--batch', dest='batch_count', metavar="batch_count",
                                  help="How many measurements to send in each API call", required=False)
        self._parser.add_argument('-o', '--origin', dest='origin', metavar="origin",
                                  help="Origin to associated with the measurements", required=False)
        self._parser.add_argument('-p', '--application-id', dest='app_id', metavar="id",
                                  help="Application Id to associate with the measurements", required=False)
        self._parser.add_argument('--skip-first-line', dest='skip_first_line', action="store_true",
                                  help="Skip header line in file")

    def handle_arguments(self, args):
        super(CSVTool, self).handle_arguments(args)

        if args.file_path is not None:
            self._file_path = args.file_path

        if args.batch_count is not None:
            self._batch_count = args.batch_count

        if args.skip_first_line is not None:
            self._skip_first_line = args.skip_first_line

        if args.origin is not None:
            self._origin = args.origin

        if args.app_id is not None:
            self._app_id = args.app_id

    def run(self, sink):
        metric = None
        value = None
        source = None
        timestamp = None
        app_id = self._app_id
        origin = self._origin
        first = self._skip_first_line
        # TODO: Only allow batch of one since the measurement API is broken for batch
        batch_count = 1
        row_count = 1
        properties = None
        if origin is not None or app_id is not None:
            properties = {}
            if origin is not None:
                properties['origin'] = self._origin
            if app_id is not None:
                properties['app_id'] = self._app_id

        with open(self._file_path) as f:
            reader = csv.reader(f)
            measurements = []
            for row in reader:
                logger.debug("row length: {0}".format(len(row)))

                if first:
                    header = ','.join(row)
                    logger.info("header: {0}".format(header))
                    first = False
                    row_count += 1
                    continue

                timestamp = int(time.time())
                if len(row) == 4:
                    metric = row[0]
                    value = row[1]
                    source = row[2]
                    timestamp = row[3]
                elif len(row) == 3:
                    metric = row[0]
                    value = row[1]
                    source = row[2]
                elif len(row) == 2:
                    metric = row[0]
                    value = row[1]
                else:
                    pass
                measurement = Measurement(metric=metric, value=value, source=source, timestamp=timestamp,
                                          properties=properties)
                logger.debug(measurement)
                measurements.append(measurement)
                if row_count % batch_count == 0:
                    logger.info("sending {0} measurements".format(len(measurements)))
                    sink.send_measurement(measurement)
                    measurements = []
                row_count += 1




#
# Copyright 2016 BMC Software, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os


class ETLCollector(object):

    def __init__(self, sink):
        self._sink = sink

    def collect(self):
        pass


class ETLTool(object):

    def __init__(self):
        self._parser = None
        self._email = None
        self._api_token = None
        self._api_host = None

    @property
    def name(self):
        return None

    @property
    def help(self):
        return None

    @property
    def email(self):
        return self._email

    @property
    def api_token(self):
        return self._api_token

    @property
    def api_host(self):
        return self._api_host

    def add_parser(self, sub_parser):
        self._parser = sub_parser.add_parser(self.name, help=self.help)
        self._parser.add_argument('-e', '--e-mail', dest='email', metavar='email')
        self._parser.add_argument('-t', '--api-token', dest='api_token', metavar='token')
        self._parser.add_argument('-a', '--api-host', dest='api_host', metavar='hostname')
        self._parser.add_argument('-s', '--sink-type', dest='sink_type', choices=['api', 'rpc', 'std'])

    def handle_arguments(self, args):
        """
        Handled the command line arguments for the tool
        :param args: Namespace object from argparse
        :return: None
        """

        if args.email is not None:
            self._email = args.email

        if args.api_token is not None:
            self._api_token = args.api_token

        if args.api_host is not None:
            self._api_host = args.api_host

    def run(self, sink):
        pass



#
# Copyright 2016 BMC Software, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from tspetl import ETLTool


class GitHubTool(ETLTool):

    def __init__(self):
        pass

    @property
    def name(self):
        return 'github'

    @property
    def help(self):
        return 'Collects data from GitHub'

    def add_parser(self, sub_parser):
        super(GitHubTool, self).add_parser(sub_parser)

    def run(self, args):
        pass



#
# Copyright 2015 BMC Software, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import absolute_import

from tspetl.etl_tool import ETLTool
from tspetl.etl_tool import ETLCollector
from tspetl.data_sink import APIDataSink
from tspetl.data_sink import StdOutDataSink
from tspetl.apache_log_tool import ApacheLogTool
from tspetl.db_tool import DBTool
from tspetl.csv_tool import CSVTool
from tspetl.github_tool import GitHubTool
from tspetl.jira_tool import JiraTool
from tspetl.log_tool import LogTool
from tspetl.salesforce_tool import SalesForceTool
from tspetl.snmp_tool import SNMPTool
from tspetl.stock_tool import StockTool
from tspetl.syslog_tool import SysLogTool
from tspetl.twitter_tool import TwitterTool
from tspetl.weather_tool import WeatherTool
from tspetl.xml_tool import XMLTool





from distutils.core import setup

setup(
    name='tspetl',
    version='0.1.2',
    url="http://github.io/boundary/meter-plugin-db",
    author='David Gwartney',
    author_email='david_gwartney@bmc.com',
    packages=['tspetl', ],
    scripts=[
    ],
    package_data={'tspetl': ['templates/*']},
    license='LICENSE',
    entry_points={
        'console_scripts': [
            'tsp-etl = tspetl.etl_cli:main',
         ],
    },
    description='TrueSight Pulse ETL Tool',
    long_description=open('README.txt').read(),
    install_requires=[
       'python-dateutil >= 2.5.2',
       'tspapi >= 0.3.4',
       'pyowm >= 2.3.0',
       'petl >= 1.1.0',
    ],
)



#
# Copyright 2016 BMC Software, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from tspapi import API


class DataSink(object):

    def __init__(self):
        pass

    def send_event(self, event):
        pass

    def send_events(self, events):
        pass

    def send_measurement(self, measurement):
        pass

    def send_measurements(self, measurements):
        pass


class APIDataSink(DataSink):

    def __init__(self, email=None, api_token=None, api_host=None):
        super(APIDataSink, self).__init__()
        self._api = API(email=email, api_token=api_token, api_host=api_host)

    def send_event(self, event):
        pass

    def send_events(self, events):
        pass

    def send_measurement(self, measurement):
        self._api.measurement_create(metric=measurement.metric,
                                     value=measurement.value,
                                     source=measurement.source,
                                     timestamp=measurement.timestamp,
                                     properties=measurement.properties)

    def send_measurements(self, measurements):
        self._api.measurement_create_batch(measurements)


class RPCDataSink(DataSink):

    def __init__(self):
        super(RPCDataSink, self).__init__()

    def send_event(self, event):
        pass

    def send_events(self, events):
        pass

    def send_measurement(self, measurement):
        pass

    def send_measurements(self, measurements):
        pass


class StdOutDataSink(DataSink):

    def __init__(self):
        super(StdOutDataSink, self).__init__()


